//
//  mmTextViewExtensions.swift
//  iRateFlyer
//
//  Created by Kevin McNeish on 1/21/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import Foundation
import UIKit

let TEXTVIEW_PLACEHOLDER_TAG = 100
let PLACEHOLDER_MARGIN = CGFloat(8.0)

class NotificationProxy: UIView {
    
    var name: String?
    weak var objectProtocol: NSObjectProtocol!
    
    func addObserverForName(name: String?, object: AnyObject?, queue: NSOperationQueue?, usingBlock: (NSNotification!) -> ()) {
        
        self.objectProtocol = NSNotificationCenter.defaultCenter().addObserverForName(name, object: object, queue: queue, usingBlock: usingBlock)
    }
    
    convenience init() {
        self.init()
    }
    
    override init(frame: CGRect) {
        super.init(frame: frame)
    }
    
    required convenience init(coder aDecoder: NSCoder) {
        self.init()
    }
    
    deinit {
        NSNotificationCenter.defaultCenter().removeObserver(self.objectProtocol, name: self.name, object: nil)
    }
}

// Adds a placeholder to UITextView
extension UITextView {
    
    var placeholderLabel: UILabel? {
        
        get {
            return self.viewWithTag(TEXTVIEW_PLACEHOLDER_TAG) as! UILabel?
        }
    }
    
    // Placeholder text
    @IBInspectable var placeholder: String? {
        
        get {
            return self.placeholderLabel?.text
        }
        
        set {
            
            // Store the placeholder text in the label
            //var placeHolderLabel = self.viewWithTag(TEXTVIEW_PLACEHOLDER_TAG) as UILabel?
            if self.placeholderLabel == nil {
                
                // Add a placeholder label to the text view
                var placeHolderLabel = self.addPlaceholderLabel(newValue!)
                
                // Register for TextView text changes
                let notificationProxy = NotificationProxy()
                notificationProxy.addObserverForName(UITextViewTextDidChangeNotification, object: self, queue: nil, usingBlock: { [unowned self] note  in
                    
                    // Respond to Text View text changes
                    var placeHolderLabel = self.viewWithTag(TEXTVIEW_PLACEHOLDER_TAG)
                    if !self.hasText() {
                        // Get the placeholder label
                        placeHolderLabel?.hidden = false
                    }
                    else {
                        placeHolderLabel?.hidden = true
                    }
                })
                self.addSubview(notificationProxy)
                
            }
            else {
                self.placeholderLabel?.text = newValue
                self.placeholderLabel?.sizeToFit()
            }
        }
    }
    
    // Add a placeholder label to the text view
    func addPlaceholderLabel(placeholderText: String) -> UILabel {
        
        // Create the label and set its properties
        var placeholderLabel = UILabel()
        placeholderLabel.text = placeholderText
        placeholderLabel.frame.origin.x = PLACEHOLDER_MARGIN
        placeholderLabel.frame.origin.y = PLACEHOLDER_MARGIN
        placeholderLabel.frame.size.width = self.bounds.size.width - (PLACEHOLDER_MARGIN * 2)
        placeholderLabel.frame.size.height = 0.0
        placeholderLabel.font = self.font
        placeholderLabel.sizeToFit()
        placeholderLabel.textColor = UIColor.lightGrayColor()
        placeholderLabel.tag = TEXTVIEW_PLACEHOLDER_TAG
        
        // Hide the label if there is text in the text view
        placeholderLabel.hidden = (count(self.text) > 0)
        
        self.addSubview(placeholderLabel)
        
        return placeholderLabel
    }
    
}